@@ -9,26 +9,14 @@ well.
 import functools
 import os
 import pathlib
-import re
 import textwrap
 import warnings
 from collections import UserDict, UserList, UserString
 from datetime import date, datetime
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    List,
-    NamedTuple,
-    Optional,
-    Sequence,
-    Set,
-    Tuple,
-    Union,
-)
+from typing import Any, Callable, Dict, List, Optional, Sequence, Set, Tuple, Union
 
 # PyConfs imports
-from pyconfs import _converters, _exceptions, readers, writers
+from pyconfs import _converters, _exceptions, _types, readers, writers
 
 
 def _dispatch_to(converter):
@@ -190,11 +178,11 @@ class Configuration(UserDict, IsConfiguration):
     ) -> None:
         """Update the configuration from a file"""
         file_path = pathlib.Path(file_path)
-        entries = readers.from_file(
+        file_format, entries = readers.from_file(
             file_path=file_path, file_format=file_format, **reader_args
         )
         self.update_from_dict(
-            entries, source=f"{file_path.resolve()} ({file_format} reader)"
+            entries, source=f"{file_path.resolve()} ({file_format.upper()} reader)"
         )
 
     def update_from_str(self, string: str, format: str, *, source=None) -> None:
@@ -206,74 +194,26 @@ class Configuration(UserDict, IsConfiguration):
         entries = readers.from_str(format, string=string)
         self.update_from_dict(entries, source=source)
 
-    def copy_section(
-        self, from_name: Union[str, List[str]], to_name: Union[str, List[str]]
-    ) -> None:
-        """Copy one section into another, nested sections can be specified as lists"""
-
-        # Get section to copy from, allow nested names
-        section = self.get(from_name)
-        if not isinstance(section, IsConfiguration):
-            raise TypeError(f"{_dotted_name(from_name)} is not a section")
-
-        # Get section to copy to, allow nested names
-        destination = self.get(to_name)
-        if not isinstance(destination, IsConfiguration):
-            raise TypeError(f"{_dotted_name(to_name)} is not a section")
-
-        # Copy section
-        destination.update_entry(
-            key=section.name_stem, value=section, source=f"Copy of {section.name}"
-        )
-
-    def copy_default_section(
-        self, default_name: Union[str, List[str]], copy_recursively: bool = False
-    ) -> None:
-        """Copy contents of one section into all other sections"""
-
-        # Get section to copy from, allow nested names
-        default = self.get(default_name)
-        if not isinstance(default, IsConfiguration):
-            raise TypeError(f"{_dotted_name(default_name)} is not a section")
-
-        # Copy section to all other sections
-        for destination in self.sections:
-            if destination.name == default.name:
-                continue
-
-            if copy_recursively:
-                destination._copy_recursively(default)
-            else:
-                destination.update_entry(
-                    key=default.name_stem,
-                    value=default,
-                    source=f"Copy of {default.name}",
-                )
-
-    def _copy_recursively(self, section: IsConfiguration) -> None:
-        """Copy the given section to all subsections of Configuration recursively"""
-        for destination in self.sections:
-            if destination.name == section.name:
-                continue
-            destination._copy_recursively(section)
-        self.update_entry(
-            key=section.name_stem, value=section, source=f"Copy of {section.name}"
-        )
-
-    def get(self, key, default=None):
+    def get(self, key, default=None, strict=False):
         """Get a value from the configuration, allow nested keys"""
         if isinstance(key, list):
             first_key, *tail_keys = key
-            value = super().get(first_key, default)
+            value = self.get(first_key, default, strict=strict)
             if tail_keys:
                 try:
-                    return value.get(list(tail_keys), default)
+                    return value.get(list(tail_keys), default, strict=strict)
                 except AttributeError:
-                    return default
+                    if strict:
+                        raise KeyError(tail_keys)
+                    else:
+                        return default
             else:
                 return value
         else:
-            return super().get(key, default)
+            if strict:
+                return self.data[key]
+            else:
+                return super().get(key, default)
 
     @property
     def name_stem(self) -> str:
@@ -408,10 +348,19 @@ class Configuration(UserDict, IsConfiguration):
     ) -> Any:
         """Replace values in an entry based on {} format strings"""
         all_vars = Variables(**self.vars, **replace_vars, default=default)
-        value = textwrap.dedent(self.data[key]) if dedent else self.data[key]
+        value = (
+            textwrap.dedent(self.get(key, strict=True))
+            if dedent
+            else self.get(key, strict=True)
+        )
 
         # Replace variables
-        replaced = value.format_map(all_vars)
+        try:
+            replaced = value.format_map(all_vars)
+        except AttributeError:
+            raise _exceptions.ConversionError(
+                f"Only strings can be replaced, not {type(value).__name__!r} "
+            ) from None
 
         # Optionally convert value to a different data type
         if converter is None:
@@ -467,64 +416,6 @@ class Configuration(UserDict, IsConfiguration):
             **writer_args,
         )
 
-    def as_named_tuple(
-        self, template: Optional[NamedTuple] = None, **other_fields: Any
-    ) -> NamedTuple:
-        """Convert Configuration to a named tuple
-
-        If a typed NamedTuple is given as a template, then the configuration
-        will be validated against that template.
-
-        Any other fields that are supplied will be added to the named tuple in
-        addition to the data in the configuration.
-        """
-        tpl_data = {**self.data, **other_fields}
-        src_map = {
-            k: None if k in self.data else f"{k}={v!r}" for k, v in tpl_data.items()
-        }
-
-        # Create a NamedTuple template based on the current data in the Configuration
-        if template is None:
-            template = NamedTuple(
-                self.name, fields=[(k, type(v)) for k, v in tpl_data.items()]
-            )
-
-        # Use NamedTuple to validate fields
-        try:
-            tpl = template(**tpl_data)
-        except TypeError as err:
-            # Rewrite the error message if fields are missing
-            message = err.args[0].replace("__new__()", f"Configuration {self.name!r}")
-            field = (re.findall(r"'([^']+)'$", message) or ["__no_field_found__"]).pop()
-            message += f" ({self._get_source_string(field, src_map.get(field))})"
-            err.args = (message, *err.args[1:])
-            raise
-
-        # Check that types are correct
-        if not hasattr(tpl, "_field_types"):
-            return tpl
-
-        for field, field_type in tpl._field_types.items():
-            if field not in tpl_data:
-                continue
-
-            try:
-                is_correct_type = isinstance(tpl_data[field], field_type)
-            except TypeError:
-                # Not all types (e.g. subscripted generics) support isinstance checks
-                continue
-
-            if not is_correct_type:
-                message = (
-                    f"Configuration {self.name} got {type(tpl_data[field]).__name__!r} "
-                    f"type for field {field}. "
-                    f"{template.__name__} requires {field_type.__name__!r} "
-                    f"({self._get_source_string(field, src_map.get(field))})"
-                )
-                raise TypeError(message)
-
-        return tpl
-
     #
     # Add converters used to convert entries to certain types
     #
@@ -631,6 +522,10 @@ class Configuration(UserDict, IsConfiguration):
         return self.as_str()
 
 
+# Add converters to other types to Configuration
+_types.add_to_class(Configuration)
+
+
 class ConfigurationList(UserList, IsConfiguration):
     """Collect a list of Configurations in one object"""
 
@@ -838,11 +733,3 @@ def _repr_toml(value: Any) -> str:
 def _is_nested(sequence) -> bool:
     """Check if the sequence contains nested lists or dictionaries"""
     return any(isinstance(s, (dict, list, IsConfiguration)) for s in sequence)
-
-
-def _dotted_name(nested_name) -> str:
-    """Convert a nested name (with list) to dotted name"""
-    if isinstance(nested_name, list):
-        return ".".join(str(name) for name in nested_name)
-    else:
-        return nested_name
